#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Time :2023/2/14 
# @Author :wsli
# @File : py_cls.py
# @Software: PyCharm

"""
1.Staticmethod和classmethod的区别?
"""
class A(object):
    def foo(self, x):
        #实例方法
        print(f"executing foo({self}, {x})")

    @classmethod
    def class_foo(cls, x):
        #类方法
        print(f"executing class_foo({cls}, {x})")

    @staticmethod
    def static_foo(x):
        #静态方法
        print(f"executing static_foo({x})")

a = A()
a.foo(1)
a.class_foo(1)
A.class_foo(1)
a.static_foo(1)
A.static_foo(1)

"""
2.Staticmethod和classmethod使用场景?
"""






#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Time :2023/3/8 
# @Author :wsli
# @File : mitsubishi_plc.py
# @Software: PyCharm
"""
输入继电器："X100","X1A0"            // 字符串为十六进制机制
输出继电器："Y100" ,"Y1A0"           // 字符串为十六进制机制
内部继电器："M100","M200"           // 字符串为十进制
锁存继电器："L100"  ,"L200"           // 字符串为十进制
报警器：   "F100", "F200"            // 字符串为十进制
边沿继电器："V100" , "V200"          // 字符串为十进制
链接继电器："B100" , "B1A0"          // 字符串为十六进制
步进继电器："S100" , "S200"          // 字符串为十进制
数据寄存器："D100", "D200"           // 字符串为十进制
链接寄存器："W100" ,"W1A0"         // 字符串为十六进制
文件寄存器："R100","R200"            // 字符串为十进制
"""
from pymelsec import Type3E, Type4E
from pymelsec.constants import DT
class Bruce_mitsubishi_plc_client(Type4E):
    def __init__(self,ip,port=5002,plc_type="Q"):
        """
        初始化三菱plc通信对象
        :param ip:
        :param port:5002-5007
        :param return_data_type: Q or
        """
        super(Bruce_mitsubishi_plc_client, self).__init__(host=ip,port=port,plc_type='Q')



    def do_connect(self):
        """
        连接
        :return:
        """
        pass

    def close(self):
        """
        关闭
        :return:
        """
        pass

    # def read(self):
    #     pass
    #
    # def write(self):
    #     pass

    def auto_close(self,address,length,data_type):
        with Type4E(host='',port=1,plc_type='') as plc:
            plc.set_access_opt(comm_type="binary")
            read_result=plc.batch_read(
                ref_device='',
                read_size=5,
                data_type=''
            )
            print(read_result)





from loguru import logger

class Bruce_loguru():
    def document(self):
        """
        官方文档
        """
        url="https://loguru.readthedocs.io/en/stable/index.html"
        return url

    def print_debug(self,data:str):
        logger.debug(data)

    def save_log(self,log_name:str,rotation,compression):
        logger.add("file_1.log", rotation="500 MB")  # Automatically rotate too big file
        logger.add("file_2.log", rotation="12:00")  # New file is created each day at noon
        logger.add("file_3.log", rotation="1 week")  # Once the file is too old, it's rotated
        logger.add("file_X.log", retention="10 days")  # Cleanup after some time
        logger.add("file_Y.log", compression="zip")  # Save some loved space



# cc=Bruce_loguru()
# cc.print_debug("123")

import logging

# 创建一个logger对象
logger = logging.getLogger('my_logger')
logger.setLevel(logging.DEBUG)

# 创建一个handler对象，用于将日志消息发送到控制台
console_handler = logging.StreamHandler()
console_handler.setLevel(logging.DEBUG)

# 创建一个formatter对象，用于设置日志消息的格式
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# 将formatter对象添加到handler对象中
console_handler.setFormatter(formatter)

# 将handler对象添加到logger对象中
logger.addHandler(console_handler)

# 记录日志消息
logger.debug('This is a debug message')
logger.info('This is an info message')
logger.warning('This is a warning message')
logger.error('This is an error message')
logger.critical('This is a critical message')


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Time :2023/3/2 
# @Author :wsli
# @File : py_str_repr.py
# @Software: PyCharm
'''
python中 str 和 repr_python repr()与str()区别总结?
str() 的输出追求明确性和可读性，输出格式要便于理解，适合用于输出内容到用户终端。
repr() 的输出追求明确性，除了对象内容，还需要展示出对象的数据类型信息，适合开发和调试阶段使用。
'''


class TestClass:
    def __init__(self, name, age):
        self.name = name
        self.age = age


    def __repr__(self):
        return 'repr: ' + self.name + ' ,' + self.age

    def __str__(self):
        return self.name + ' ,' + self.age




tt = TestClass('tony', '23')
print(str(tt))
print(repr(tt))


"""
└── django // 根目录
    ├── apps
    ├── conf
    ├── contrib
    ├── core
    ├── db
    ├── dispatch
    ├── forms
    ├── http
    ├── middleware
    ├── template
    ├── templatetags
    ├── test
    ├── urls
    ├── utils
    └── views
apps：与 Django 中定义的 app 相关的逻辑，Django中app即是在settings文件中配置的 INSTALLED_APPS。
conf：公共配置信息相关的目录，存放一些模版文件与 settings默认配置。
contrib：是 Django 内置的强大的功能包，相当于 Django 中的各种子模块，里面有大量额外提供的、或者增加 Django 功能的库及函数，比如 contrib 目录下的 auth 模块提供了与Django的用户验证相关的框架，admin模块提供了与自动化站点管理相关的后台功能。
core：Django的核心功能目录，核心的功能都位于这个目录下， Django 下各种命令的使用如 django startproject && python manage.py xxx 等核心命令的使用都在此目录下。
db：数据库连接、模型Model定义以及 ORM 相关的逻辑。Django可以兼容很多数据库，包括MySQL、Oracle，PgSql等等，通过这个模块可以屏蔽不同数据库之间底层的差异，实现统一接口。
dispatch：处理信号相关的模块，在 Django 中很多地方都有用到信号，比如生成数据库迁移文件时。
forms：表单处理，主要用于与前端表单组件交互的封装。
http：Http请求和应答等与网络请求相关的模块。
middleware：内置的中间件模块，有很多方便可直接使用的中间件。
Template&&Templatetags：Django的模版引擎。
test：单元测试相关的模块。
urls：处理路由相关的模块。
utils：提供了很多使用的小工具类，比如懒加载类就在这个目录下。
views：视图处理的模块

"""


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Time :2023/2/13 
# @Author :wsli
# @File : py_thread.py
# @Software: PyCharm
import inspect
import ctypes
import threading
import time

class Bruce_threading():
    def __init__(self,Thread_list:list,stop_time:int=-1):
        """
        初始化
        :param Thread_list:
        :param stop_time: 休眠多少秒停止所有线程运行
        """
        #线程列表
        self.Thread=Thread_list
        self._append_stop(stop_time)
    def start(self):
        """
        开启所有线程
        :return:
        """
        for i in self.Thread:
            i.setDaemon(True)
            i.start()
        for i in self.Thread:
            i.join()

    def _async_raise(self,tid, exctype):
        """raises the exception, performs cleanup if needed"""
        tid = ctypes.c_long(tid)
        if not inspect.isclass(exctype):
            exctype = type(exctype)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        if res == 0:
            raise ValueError("invalid thread id")
        elif res != 1:
            # """if it returns a number greater than one, you're in trouble,
            # and you should call it again with exc=NULL to revert the effect"""
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")
    def _append_stop(self,sleep_time):
        if sleep_time !=-1:
            t1=threading.Thread(target=self.stop_thread,args=(self.Thread,sleep_time,))
            self.Thread.append(t1)
    def stop_thread(self,thread_list,sleep_time):
        """
        停止线程运行
        :param thread:
        :return:
        """
        time.sleep(sleep_time)
        for thread in thread_list:
            try:
                self._async_raise(thread.ident, SystemExit)
            except Exception as e:
                print(e)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# @Time :2023/2/25 
# @Author :wsli
# @File : py_mat.py
# @Software: PyCharm
from matplotlib import pyplot as plt
import numpy as np


class Bruce_li_matplotlib():
    def is_ch(self,data):
        """
        检验是否含有中文字符
        :param data:
        :return:
        """
        for _char in data:
            if '\u4e00' <= _char <= '\u9fa5':
                return True
        return False

    def temp_font_setting(self,ch_type="中文黑体"):
        """
        临时更改字体设置
        :return:
        """
        ch_type_data=None
        if ch_type=="中文黑体" or ch_type=="黑体":
            ch_type_data="SimHei"
        elif ch_type=="中文楷体" or ch_type=="楷体":
            ch_type_data = "Kaiti"
        elif ch_type=="中文隶书" or ch_type=="隶书":
            ch_type_data = "LiSu"
        elif ch_type == "中文仿宋" or ch_type == "仿宋":
            ch_type_data = "FangSong"
        elif ch_type == "中文幼圆" or ch_type == "幼圆":
            ch_type_data = "FangSong"
        elif ch_type == "华文宋体":
            ch_type_data = "STSong"
        plt.rcParams["font.sans-serif"]=[ch_type_data]
        #当字体设置为中文时,负号失效
        plt.rcParams["axes.unicode_minus"] = False

    def set_title(self,title,fontsize=16.0):
        plt.title(title,fontsize=fontsize)

    def set_x(self,name,fontsize=16.0):
        plt.xlabel(name,fontsize=fontsize)

    def set_y(self,name,fontsize=16.0):
        plt.ylabel(name,fontsize=fontsize)

    def set_x_scale(self,rottation,color):
        """
        设置x轴刻度
        :param rottation: 旋转
        :param color: 颜色
        :return:
        """
        plt.xticks(rottation=rottation,color=color)

    def set_y_scale(self):
        plt.yticks()

    def legend(self):
        """
        图例（哪个线段表示的什么含义）
        :return:
        """
        pass

    def range_default_table(self,title:str="",
                            x_name:str="",
                            y_name:str="",
                            linewidth:float=2.0
                            ):
        """
        绘制线性图表
        :return:
        """
        x = np.arange(-50, 51)
        y = x ** 2

        x1 = np.arange(-50, 51)
        y1 = x ** 1
        if self.is_ch(title):
            self.temp_font_setting()

        self.set_title(title)
        #x轴名称
        self.set_x(x_name)
        #y轴名称
        self.set_y(y_name)
        #self.set_x_scale()

        # 绘制线性图表

        plt.plot(x, y,linewidth)

        # plt.plot(x1, y1, linewidth)
        plt.plot(x,label="123")
        plt.plot(x1, label="1234")
        plt.legend()
        plt.show()

cc=Bruce_li_matplotlib()
# list1={
#     "title":"",
#     "data":[
#         {
#         "x":np.arange(-50, 51),
#         "y":np.arange(-50, 51)** 2,
#         "x_name":"",
#         "x_font_size":"",
#         "y_name":"",
#         "y_font_size":"",
#         "line_width":""
#         }
#     ]
#
# }
cc.range_default_table()


__author__ = "Burgess Zheng"
# !/usr/bin/env python
# -*- coding:utf-8 -*-
'''
简单工厂模式
simple Factory Method
'''


class Shape(object):
    '''
    父类
    '''

    def draw(self):
        raise NotImplementedError


class Circle(Shape):
    '''
    Shape子类
    '''

    def draw(self):
        print('draw circle')


class Rectangle(Shape):
    '''
    Shape的子类
    '''

    def draw(self):
        print('draw Rectangle')


class ShapeFactory(object):
    '''
    工厂模式：暴露给用户去调用的，
    用户可通过该类进行选择Shape的子类进行实例化
    '''

    def create(self, shape):
        if shape == 'Circle':
            return Circle()
        elif shape == 'Rectangle':
            return Rectangle()
        else:
            return None


fac = ShapeFactory()  # 实例化工厂类
obj = fac.create('Circle')  # 实例化Shape的Circle子类
obj.draw()


#_*_coding:utf-8_*_
import socket,subprocess
ip_port=('127.0.0.1',8089)
s=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

s.bind(ip_port)
s.listen(5)

while True:
    conn,addr=s.accept()
    print('客户端',addr)
    while True:
        msg=conn.recv(1024)
        if not msg:
            break
        res=subprocess.Popen(msg.decode('utf-8'),shell=True,\
                            stdin=subprocess.PIPE,\
                         stderr=subprocess.PIPE,\
                         stdout=subprocess.PIPE)
        err=res.stderr.read()
        if err:
            ret=err
        else:
            ret=res.stdout.read()
        data_length=len(ret)
        conn.send(str(data_length).encode('utf-8'))
        data=conn.recv(1024).decode('utf-8')
        print("服务端获取数据为",data)
        if data == 'recv_ready':
            conn.sendall(ret)
    conn.close()



# encoding: utf-8
import time

import pymysql
from dbutils.pooled_db import PooledDB

class Bruce_mysql():
    # 初始化
    def __init__(self,dbName, host="localhost",port=3306, user="root", passwd="123456"):
        self.host = host
        self.port=port
        self.user = user
        self.passwd = passwd
        self.dbName = dbName
        self.connet()

    # 连接数据库，需要传数据库地址、用户名、密码、数据库名称，默认设置了编码信息
    def connet(self):
        try:
            self.db = pymysql.connect(host=self.host,
                                      port=self.port,
                                      user=self.user,
                                      password=self.passwd,
                                      db=self.dbName,
                                      use_unicode=True,
                                      charset='utf8')
            self.cursor = self.db.cursor()
        except Exception as e:
            return e

    # 关闭数据库连接
    def close(self):
        try:
            self.cursor.close()
            self.db.close()
        except Exception as e:
            return e

    # 查询操作，查询单条数据
    def get_one(self, sql):
        # res = None
        try:
            self.connet()
            self.cursor.execute(sql)
            res = self.cursor.fetchone()
            self.close()
        except Exception:
            res = None
        return res

    # 查询操作，查询多条数据
    def get_all(self, sql):
        # res = ()
        try:
            self.connet()
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
            self.close()
        except Exception:
            res = ()
        return res

    # 查询数据库对象
    def get_all_obj(self, sql, tableName, *args):
        resList = []
        fieldsList = []
        try:
            if (len(args) > 0):
                for item in args:
                    fieldsList.append(item)
            else:
                fieldsSql = "select COLUMN_NAME from information_schema.COLUMNS where table_name = '%s' and table_schema = '%s'" % (
                    tableName, self.dbName)
                fields = self.get_all(fieldsSql)
                for item in fields:
                    fieldsList.append(item[0])

            # 执行查询数据sql
            res = self.get_all(sql)
            for item in res:
                obj = {}
                count = 0
                for x in item:
                    obj[fieldsList[count]] = x
                    count += 1
                resList.append(obj)
            return resList
        except Exception as e:
            return e

    # 数据库插入、更新、删除操作
    def insert(self, sql):
        return self.__edit(sql)

    def update(self, sql):
        return self.__edit(sql)

    def delete(self, sql):
        return self.__edit(sql)

    def __edit(self, sql):
        # count = 0
        try:
            self.connet()
            count = self.cursor.execute(sql)
            self.db.commit()
            self.close()
        except Exception as e:
            self.db.rollback()
            count = 0
        return count


    def run_sql(self,sql):
        # res = None
        try:
            self.connet()
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
            self.close()
        except Exception:
            res = None
        return res
    def cat_max_connections(self):
        """
        看当前的最大连接数
        默认是151个,最大连接数可以修改为16384
        """
        sql="show variables like 'max_connections';"
        data=self.run_sql(sql)
        return data

    def cat_variables(self):
        """
        查看数据库所有配置
        """
        sql="show global variables;"
        data=self.run_sql(sql)
        return data

    def repair_table(self,table_name):
        """
        修复表
        """
        sql_data="REPAIR TABLE `{table_name}`;".format(table_name=table_name)
        data=self.run_sql(sql_data)
        return data

class Bruce_mysql_pool:
    def __init__(self,host='localhost', port=3306,password="123456"):
        # 创建连接池
        self.host = host
        self.port = port
        self.password = password
        self.pool = self._create_pool()
    def _create_pool(self):
        return PooledDB(
            creator=pymysql, # 使用数据库的模块
            maxconnections=5,  # 设置最大连接数量
            mincached=2,  # 设置初始空闲连接数量
            maxcached=3,  # 连接池中最多可以存放的空闲连接数量
            blocking=True,  # 连接池中没有空闲连接后设置是否等待，True等待，False不等待
            host='localhost',  # 数据库主机地址
            port=3306,  # 数据库端口
            ping=0,# #检查服务是否可用
            # # ping MySQL服务端，检查是否服务可用。# 如：0 = None = never, 1 = default = whenever it is requested, 2 = when a cursor is created, 4 = when a query is executed, 7 = always
            user='root',  # 数据库用户名
            password='123456',  # 数据库密码
            #database='test'  # 默认连接的数据库
        )

    def create_mysql_client(self):
        """
        连接池中拿一个连接，自动回收连接
        """
        # 获取Mysql连接
        conn = Bruce_mysql(self.pool)
        return conn


class Mysql_mock_error:
    def mock_client_max(self, host='localhost', port=3306, user="root", max_connections=148, password="123456"):
        """
        错误示范
        模拟redis客户端连接数超过10000
        cc=Redis_mock_error()
        cc.mock_client_max()
        """
        connections = []
        try:
            for _ in range(max_connections):
                print(f"Creating connection {len(connections) + 1}/{max_connections}")
                connection =  pymysql.connect(host=host,
                                      port=port,
                                      user=user,
                                      password=password,
                                      use_unicode=True,
                                      charset='utf8')

                connection.commit()
                connections.append(connection)
            time.sleep(10000)
        except pymysql.err.OperationalError as eroor:
            print("mysql连接数已经达到最大值,不能连接了", eroor)
        except Exception as e:
            print("e", e)

#
# Mysql_mock_error().mock_client_max()



import os
class Bruce_generate:
    def __init__(self,BASE_DIR:str):
        #self.FILE_PATH=os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

        self.FILE_PATH = str(BASE_DIR)
    def make_one(self):
        """
        一键生成
        """
        self.make_run_sh()
        self.make_run_bat()
        self.make_migrate_bat()
        self.make_migrate_sh()

    def make_migrate_bat(self)->bool:
        """
        生成django项目windows的迁移文件
        """
        try:
            data="python manage.py makemigrations  && python manage.py migrate"
            file_name="migrate.bat"
            # 返回脚本绝对路径
            #path=os.path.abspath(__file__)
            # 返回脚本上一层目录路径
            #root_path1 = os.path.dirname(os.path.abspath(__file__))
            # 返回脚本上两层目录路径
            #root_path2 = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
            with open(self.FILE_PATH+"\\"+file_name, 'w') as file:
                file.write(data)
            return True
        except Exception as e:
            return False

    def make_migrate_sh(self):
        """
        生成django项目liunx的迁移文件
        """
        try:
            data = "python manage.py makemigrations  && python manage.py migrate"
            file_name = "migrate.sh"
            with open(self.FILE_PATH + "\\" + file_name, 'w') as file:
                file.write(data)
            return True
        except Exception as e:
            return False

    def make_run_bat(self,port=8000):
        """
        生成django项目windows的bat运行文件
        """
        try:
            data = "python manage.py runserver 0.0.0.0:8000"
            file_name = "run.bat"
            with open(self.FILE_PATH + "\\" + file_name, 'w') as file:
                file.write(data)
            return True
        except Exception as e:
            return False
        pass
    def make_run_sh(self,port=8000):
        """
        生成django项目liunx的sh运行文件
        """
        data="python manage.py runserver 0.0.0.0:8000"
        file_name = "run.sh"
        try:
            data = "python manage.py runserver 0.0.0.0:8000"
            file_name = "run.sh"
            with open(self.FILE_PATH + "\\" + file_name, 'w') as file:
                file.write(data)
            return True
        except Exception as e:
            return False





#_*_coding:utf-8_*_
import socket,time
s=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
res=s.connect_ex(('127.0.0.1',8089))

while True:
    msg=input('>>: ').strip()
    if len(msg) == 0:continue
    if msg == 'quit':break

    s.send(msg.encode('utf-8'))
    length=int(s.recv(1024).decode('utf-8'))
    s.send('recv_ready'.encode('utf-8'))
    send_size=0
    recv_size=0
    data=b''
    while recv_size < length:
        data+=s.recv(1024)
        recv_size+=len(data)


    print("data",data)



import setuptools


description="""
武器库
"""
requirements = [
    "python-snap7==1.3",
    "docker==6.0.1",
    "modbus_tk==1.1.2",
    "pyModbusTCP==0.2.0",
    "pyinstaller==5.7.0",   #将 Python 程序转换成独立的执行文件（跨平台）
    "sqlalchemy==2.0.4",    #python SQL 工具以及对象关系映射工具
    "pony==0.7.16",         #提供面向生成器的 SQL 接口的 ORM
    "gitpython==3.1.30",
    "requests==2.28.2",
    "pymysql==1.0.2",
    "redis==4.5.1",
    "pywin32==305",         # 针对 Windows 的Python 扩展
    "rpyc==5.3.1",           #RPC 服务器
    "paho-mqtt==1.6.1",      #MQTT协议通讯
    "websocket==0.2.1",         #websocket协议
    "websocket-client==1.6.4", #websocket协议
    "DBUtils==3.0.3"
    # "ftplib",               #FTP协议
    # "pika==1.3.1",          #RabbitMQ
    # "alembic==1.9.4",        #数据迁移
    # "matplotlib==3.4.3"
    #xlwt / xlrd     读写 Excel 文件的数据和格式信息
    #BeautifulSoup   以 Python 风格的方式来对 HTML 或 XML 进行迭代，搜索和修改
    #envoy           比 Python subprocess 模块更人性化。
    #NumPy           进行科学计算的基础包
    #Pandas          提供高性能，易用的数据结构和数据分析工具
    #matplotlib      一个 Python 2D 绘图库
    #psutil          一个跨平台进程和系统工具模块
    #WinPython       Windows 7/8 系统下便携式开发环境
    #Six             Python 2 和 3 的兼容性工具
    #celery  一个异步任务队列/作业队列，基于分布式消息传递

]       # 自定义工具中需要的依赖包
 
setuptools.setup(
    name="Bruce_li_one",       # 自定义工具包的名字
    version="0.0.22",             # 版本号
    author="Bruce_li123",           # 作者名字
    author_email="lws__xinlang@sina.com",  # 作者邮箱
    description=description, # 自定义工具包的简介
    license='MIT-0',           # 许可协议
    url="",              # 项目开源地址
    packages=setuptools.find_packages(),  # 自动发现自定义工具包中的所有包和子包
    install_requires=requirements,  # 安装自定义工具包需要依赖的包
    python_requires='>=3.5',         # 自定义工具包对于python版本的要求
    # 项目相关额外连接，如代码仓库，文档地址等
    project_urls = {
       "Documentation": "https://bruce-li-one-docs.readthedocs.io/en/latest/Cpp/index.html",
    }
)




class Bruce_generate:
    def make_migrate_bat(self):
        """
        生成flask项目windows的迁移文件
        """
        data="flask db migrate && flask db upgrade"
        file_name = "migrate_bat"
        pass
    def make_migrate_sh(self):
        """
        生成flask项目liunx的迁移文件
        """
        data="flask db migrate && flask db upgrade"
        file_name = "migrate_sh"
        pass
    def make_run_bat(self,host="",port=8000):
        """
        生成flask项目windows的bat运行文件
        """
        data="flask  --app app.py run -h 0.0.0.0 -p 8000 --debug"
        file_name = "run.bat"
        pass
    def make_run_sh(self,host="",port=8000):
        """
        生成flask项目liunx的sh运行文件
        """
        data="flask  --app app.py run -h 0.0.0.0 -p 8000 --debug"
        file_name="run.sh"
        pass